#include <stdlib.h>
#include "ldap_plugin.h"
#include "interface.h"
#include "support.h"
#include <gnome.h>

GtkWidget *wnd_options = NULL;
ldap_connection *ldapconn = NULL;

void messageBox(GtkMessageType msg_type, GtkButtonsType btn_type, char *message, ...)
{
	va_list arglist;
	char buffer[1024];
	va_start(arglist, message);
	vsprintf(buffer, message, arglist);
	GtkWidget *dialog = gtk_message_dialog_new ((GtkWindow *)wnd_options, GTK_DIALOG_DESTROY_WITH_PARENT, msg_type, btn_type, buffer);
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
	va_end(arglist);
}

int check_entries() {
	GtkEntry *txt_server = (GtkEntry *)lookup_widget(wnd_options, "txt_server");
	GtkEntry *txt_port = (GtkEntry *)lookup_widget(wnd_options, "txt_port");
	GtkEntry *txt_binddn = (GtkEntry *)lookup_widget(wnd_options, "txt_binddn");
	GtkEntry *txt_passwd = (GtkEntry *)lookup_widget(wnd_options, "txt_passwd");
	GtkEntry *txt_searchbase = (GtkEntry *)lookup_widget(wnd_options, "txt_searchbase");
	GtkEntry *txt_filter = (GtkEntry *)lookup_widget(wnd_options, "txt_filter");
	if (!strlen(gtk_entry_get_text(txt_server))) {
		//No server given
		messageBox(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "No Server given");
		return 1;
	}
	if (!strlen(gtk_entry_get_text(txt_port))) {
		//No server given
		messageBox(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "No Port given");
		return 1;
	}
	if (!strlen(gtk_entry_get_text(txt_binddn))) {
		//No server given
		messageBox(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "No Binddn given");
		return 1;
	}
	if (!strlen(gtk_entry_get_text(txt_passwd))) {
		//No server given
		messageBox(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "No Password given");
		return 1;
	}
	if (!strlen(gtk_entry_get_text(txt_searchbase))) {
		//No server given
		messageBox(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "No Searchbase given");
		return 1;
	}
	return 0;
}

GtkWidget* open_option_window(sync_pair *pair, connection_type type)
{
	char port[1024];

	//Create and show the Options window
	wnd_options = create_wnd_options();
	gtk_widget_show (wnd_options);

	//Create a new ldapconnection
	ldapconn = malloc(sizeof(ldap_connection));
	ldapconn->debug_level = 0;
	sprintf(ldapconn->statefile, "%s/%sldap", sync_get_datapath(pair), (type==CONNECTION_TYPE_LOCAL?"local":"remote"));

	if (!load_ldap_state(ldapconn)) {
		sprintf(port, "%i", ldapconn->serverport);
		gtk_entry_set_text((GtkEntry *)lookup_widget(wnd_options, "txt_server"), ldapconn->servername);
		gtk_entry_set_text((GtkEntry *)lookup_widget(wnd_options, "txt_port"), port);
		gtk_entry_set_text((GtkEntry *)lookup_widget(wnd_options, "txt_binddn"), ldapconn->binddn);
		gtk_entry_set_text((GtkEntry *)lookup_widget(wnd_options, "txt_passwd"), ldapconn->pwd);
		gtk_entry_set_text((GtkEntry *)lookup_widget(wnd_options, "txt_searchbase"), ldapconn->searchbase);
		gtk_entry_set_text((GtkEntry *)lookup_widget(wnd_options, "txt_filter"), ldapconn->filter);
		if (ldapconn->scope == LDAP_SCOPE_ONELEVEL) {
			gtk_entry_set_text((GtkEntry *)((GtkCombo *)lookup_widget(wnd_options, "cmb_scope"))->entry, "One Level");
		} else {
			gtk_entry_set_text((GtkEntry *)((GtkCombo *)lookup_widget(wnd_options, "cmb_scope"))->entry, "Sub Tree");
		}

		switch (ldapconn->debug_level) {
			case 0:
				gtk_entry_set_text((GtkEntry *)((GtkCombo *)lookup_widget(wnd_options, "cmb_debug"))->entry, "Errors Only");
				break;
			case 1:
				gtk_entry_set_text((GtkEntry *)((GtkCombo *)lookup_widget(wnd_options, "cmb_debug"))->entry, "Errors and Warnings");
				break;
			case 2:
				gtk_entry_set_text((GtkEntry *)((GtkCombo *)lookup_widget(wnd_options, "cmb_debug"))->entry, "Information");
				break;
			case 3:
				gtk_entry_set_text((GtkEntry *)((GtkCombo *)lookup_widget(wnd_options, "cmb_debug"))->entry, "Debug");
				break;
			case 4:
				gtk_entry_set_text((GtkEntry *)((GtkCombo *)lookup_widget(wnd_options, "cmb_debug"))->entry, "Full Debug");
				break;
		}
	}

	return(wnd_options);
}

void on_btn_ok_clicked (GtkButton *button, gpointer user_data)
{
	GtkEntry *txt_server = (GtkEntry *)lookup_widget(wnd_options, "txt_server");
	GtkEntry *txt_port = (GtkEntry *)lookup_widget(wnd_options, "txt_port");
	GtkEntry *txt_binddn = (GtkEntry *)lookup_widget(wnd_options, "txt_binddn");
	GtkEntry *txt_passwd = (GtkEntry *)lookup_widget(wnd_options, "txt_passwd");
	GtkEntry *txt_searchbase = (GtkEntry *)lookup_widget(wnd_options, "txt_searchbase");
	GtkEntry *txt_filter = (GtkEntry *)lookup_widget(wnd_options, "txt_filter");

	if (check_entries()) {
		return;
	}

	ldapconn->servername = strdup(gtk_entry_get_text(txt_server));
	ldapconn->serverport = atoi(gtk_entry_get_text(txt_port));
	ldapconn->binddn =  strdup(gtk_entry_get_text(txt_binddn));
	ldapconn->pwd =  strdup(gtk_entry_get_text(txt_passwd));
	ldapconn->searchbase =  strdup(gtk_entry_get_text(txt_searchbase));
	ldapconn->filter =  strdup(gtk_entry_get_text(txt_filter));
	if (!strcmp(gtk_entry_get_text((GtkEntry *)((GtkCombo *)lookup_widget(wnd_options, "cmb_scope"))->entry), "One Level")) {
		ldapconn->scope = LDAP_SCOPE_ONELEVEL;
	} else {
		ldapconn->scope = LDAP_SCOPE_SUBTREE;
	}

	if (!strcmp(gtk_entry_get_text((GtkEntry *)((GtkCombo *)lookup_widget(wnd_options, "cmb_debug"))->entry), "Errors Only")) {
		ldapconn->debug_level = 0;
	}
	if (!strcmp(gtk_entry_get_text((GtkEntry *)((GtkCombo *)lookup_widget(wnd_options, "cmb_debug"))->entry), "Errors and Warnings")) {
		ldapconn->debug_level = 1;
	}
	if (!strcmp(gtk_entry_get_text((GtkEntry *)((GtkCombo *)lookup_widget(wnd_options, "cmb_debug"))->entry), "Information")) {
		ldapconn->debug_level = 2;
	}
	if (!strcmp(gtk_entry_get_text((GtkEntry *)((GtkCombo *)lookup_widget(wnd_options, "cmb_debug"))->entry), "Debug")) {
		ldapconn->debug_level = 3;
	}
	if (!strcmp(gtk_entry_get_text((GtkEntry *)((GtkCombo *)lookup_widget(wnd_options, "cmb_debug"))->entry), "Full Debug")) {
		ldapconn->debug_level = 4;
	}

	save_ldap_state(ldapconn);

	gtk_widget_destroy(wnd_options);
}

void on_btn_test_clicked (GtkButton *button, gpointer user_data)
{
	GtkEntry *txt_server = (GtkEntry *)lookup_widget(wnd_options, "txt_server");
	GtkEntry *txt_port = (GtkEntry *)lookup_widget(wnd_options, "txt_port");
	GtkEntry *txt_binddn = (GtkEntry *)lookup_widget(wnd_options, "txt_binddn");
	GtkEntry *txt_passwd = (GtkEntry *)lookup_widget(wnd_options, "txt_passwd");
	GtkEntry *txt_searchbase = (GtkEntry *)lookup_widget(wnd_options, "txt_searchbase");
	GtkEntry *txt_filter = (GtkEntry *)lookup_widget(wnd_options, "txt_filter");
	GtkImage *img_connection = (GtkImage *)lookup_widget(wnd_options, "img_connection");
	GtkImage *img_authenticate = (GtkImage *)lookup_widget(wnd_options, "img_authenticate");
	GtkImage *img_search = (GtkImage *)lookup_widget(wnd_options, "img_search");
	GtkImage *img_write = (GtkImage *)lookup_widget(wnd_options, "img_write");
	GtkImage *img_evolution = (GtkImage *)lookup_widget(wnd_options, "img_evolution");
	LDAP *ld = NULL;
	int ldapResult;
	LDAPMessage *res, *res2;
	char *attrs[] = {"cn", NULL};
	BerElement *berptr = NULL;
	char *attribute = NULL;
	char **values = NULL;
	int i = 0, found = 0;
	LDAPMod **data;
	char dn[1024];

	//Check entries
	if (check_entries()) {
		return;
	}

	//Check connection
	ld = ldap_open(gtk_entry_get_text(txt_server), atoi(gtk_entry_get_text(txt_port)));
	if (!ld) {
		messageBox(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "Could not connect to %s on port %s", gtk_entry_get_text(txt_server), gtk_entry_get_text(txt_port) );
		gtk_image_set_from_stock(img_connection, "gtk-dialog-error", GTK_ICON_SIZE_BUTTON);
		return;
	} else {
		gtk_image_set_from_stock(img_connection, "gtk-apply", GTK_ICON_SIZE_BUTTON);
	}

	//Check Authentication
	ldapResult = ldap_simple_bind_s(ld, gtk_entry_get_text(txt_binddn), gtk_entry_get_text(txt_passwd));
	if (ldapResult != LDAP_SUCCESS) {
		messageBox(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "Unable to bind to server.\nServer said: %s", ldap_err2string(ldapResult));
		gtk_image_set_from_stock(img_authenticate, "gtk-dialog-error", GTK_ICON_SIZE_BUTTON);
		return;
	} else {
		gtk_image_set_from_stock(img_authenticate, "gtk-apply", GTK_ICON_SIZE_BUTTON);
	}

	//Try to search
	ldapResult = ldap_search_s(ld, gtk_entry_get_text(txt_binddn), LDAP_SCOPE_ONELEVEL, gtk_entry_get_text(txt_filter), attrs, 0, &res);
	if (ldapResult != LDAP_SUCCESS) {
		messageBox(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "Unable to search.\nServer said: %s", ldap_err2string(ldapResult));
		gtk_image_set_from_stock(img_search, "gtk-dialog-error", GTK_ICON_SIZE_BUTTON);
		return;
	} else {
		gtk_image_set_from_stock(img_search, "gtk-apply", GTK_ICON_SIZE_BUTTON);
	}

	//Try to add a invalid dn
	data = g_malloc0(1024 * sizeof(LDAPMod *));

	data[i] = g_malloc0(sizeof(LDAPMod));
	data[i]->mod_values = g_malloc0(2 * sizeof(char *));
	data[i]->mod_type = "cn";
	data[i]->mod_values[0] = "65af6f22fhfnbvJasdhasud2374";
	data[i]->mod_values[1] = NULL;
	data[i]->mod_op = LDAP_MOD_ADD;

	i++;
	data[i] = g_malloc0(sizeof(LDAPMod));
	data[i]->mod_values = g_malloc0(2 * sizeof(char *));
	data[i]->mod_type = "sn";
	data[i]->mod_values[0] = "test";
	data[i]->mod_values[1] = NULL;
	data[i]->mod_op = LDAP_MOD_ADD;

	i++;
	data[i] = g_malloc0(sizeof(LDAPMod));
	data[i]->mod_values = g_malloc0(4 * sizeof(char *));
	data[i]->mod_type = "objectClass";
	data[i]->mod_values[0] = "top";
	data[i]->mod_values[1] = "person";
	data[i]->mod_values[2] = "organizationalPerson";
	data[i]->mod_values[3] = NULL;
	data[i]->mod_op = LDAP_MOD_ADD;
	data[i + 1] = NULL;

	sprintf(dn, "cn=65af6f22fhfnbvJasdhasud2374,%s", gtk_entry_get_text(txt_searchbase));

	if (ldapResult = ldap_add_s(ld, dn, data)) {
		messageBox(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "Unable to add entry.\nServer said: %s", ldap_err2string(ldapResult));
		gtk_image_set_from_stock(img_write, "gtk-dialog-error", GTK_ICON_SIZE_BUTTON);
		return;
	}

	if (ldapResult = ldap_delete_s(ld, dn)) {
		messageBox(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "Unable to delete entry.\nServer said: %s", ldap_err2string(ldapResult));
		gtk_image_set_from_stock(img_write, "gtk-dialog-error", GTK_ICON_SIZE_BUTTON);
		return;
	} else {
		gtk_image_set_from_stock(img_write, "gtk-apply", GTK_ICON_SIZE_BUTTON);
	}

	//Check evolution compliance
	attrs[0] = "attributeTypes";
	attrs[1] = NULL;
	if (ldapResult = ldap_search_s(ld, "cn=Subschema", LDAP_SCOPE_BASE, "(objectClass=*)", attrs, 0, &res)) {
		messageBox(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "Unable to search for subschemaSubentry.\nServer said: %s", ldap_err2string(ldapResult));
		gtk_image_set_from_stock(img_evolution, "gtk-dialog-error", GTK_ICON_SIZE_BUTTON);
		return;
	}

	res2 = ldap_first_entry(ld, res);
	if (!res2) {
		messageBox(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "No entry returned");
		gtk_image_set_from_stock(img_evolution, "gtk-dialog-error", GTK_ICON_SIZE_BUTTON);
		return;
	}

	attribute = ldap_first_attribute(ld, res2, &berptr);
	if (!attribute) {
		gtk_image_set_from_stock(img_evolution, "gtk-dialog-error", GTK_ICON_SIZE_BUTTON);
		return;
	}
	values =  ldap_get_values(ld, res2, attribute);
	if (!values) {
		gtk_image_set_from_stock(img_evolution, "gtk-dialog-error", GTK_ICON_SIZE_BUTTON);
		return;
	}
	for(i = 0; values[i]; i++) {
		if (!strcmp(values[i], "( 1.3.6.1.4.1.8506.1.2.8 NAME 'spouseName' SUP name SINGLE-VALUE )")) {
			found = 1;
			break;
		}
	}

	if (!found) {
		messageBox(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "Could not find the attributes required for evolution");
		gtk_image_set_from_stock(img_evolution, "gtk-dialog-error", GTK_ICON_SIZE_BUTTON);
		return;
	} else {
		gtk_image_set_from_stock(img_evolution, "gtk-apply", GTK_ICON_SIZE_BUTTON);
	}
}

void
on_btn_cancel_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	printf("test1\n");
	gtk_widget_destroy(wnd_options);
	wnd_options = NULL;
}

void
on_wnd_options_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
	printf("test3\n");
	sync_plugin_window_closed();
}
