#include "ldap_plugin.h"

/*Connect and bind to a ldap server*/
int ldap_start(ldap_connection *conn)
{
	ldap_debug(conn, 2, "Connecting to %s", conn->servername);
	conn->ld = ldap_init(conn->servername, conn->serverport);

	if (ldap_simple_bind_s(conn->ld, conn->binddn, conn->pwd)) {
		ldap_debug(conn, 2, "Unable to connect and bind to %s as %s", conn->servername, conn->binddn);
		return 1;
	}
	ldap_debug(conn, 3, "end: ldap_start");
	return 0;
}
/*
struct data_entry *ldap_get_entry(ldap_connection *conn, char *uid)
{
	LDAPMessage *res, *res2;
	struct data_entry *entry = NULL;
	char *attrs[] = {"cn", "entryUUID", "modifyTimestamp", NULL};
	entry = g_malloc0(sizeof(struct data_entry));
	char filter[1024];
	uid = quoted_decode(uid);

	ldap_debug(conn, 2, "Loading single head data for %s", ldap_explode_dn(uid, 0)[0]);

	//Search all entries that apply to the filter. get entryUUID and modifyTimestamp
	sprintf(filter, "(%s)", ldap_explode_dn(uid, 0)[0]);
	if (ldap_search_s(conn->ld, conn->searchbase, LDAP_SCOPE_ONELEVEL, filter, attrs, 0, &res)) {
		ldap_debug(conn, 0, "Unable to search on %s", conn->searchbase);
		return NULL;
	}

	res2 = ldap_first_entry(conn->ld, res);
	if (!res2) {
		ldap_debug(conn, 2, "No entries found");
		return NULL;
	}

	do {
		if (!strcmp(ldap_get_dn(conn->ld, res2), uid)) {
			entry->entryUUID = (ldap_get_values(conn->ld, res2, "entryUUID"))[0];
			entry->modifyTimestamp = (ldap_get_values(conn->ld, res2, "modifyTimestamp"))[0];
			entry->uid = quoted_encode(ldap_get_dn(conn->ld, res2));
			ldap_debug(conn, 2, "Loaded entry: %s, %s, %s", entry->entryUUID, entry->modifyTimestamp, entry->uid);
			break;
		}
		res2 = ldap_next_entry(conn->ld, res2);
	} while (res2);

	ldap_debug(conn, 3, "done: ldap_get_entry");
	return entry;
}*/

/*load the head data from the ldap server*/
GList *load_ldap_entries(ldap_connection *conn)
{
	LDAPMessage *res, *res2;
	GList *headlist = NULL;
	struct data_entry *entry = NULL;
	char *attrs[] = {"cn", "entryUUID", "modifyTimestamp", NULL};

	ldap_debug(conn, 2, "Loading head data from ldap");

	//Search all entries that apply to the filter. get entryUUID and modifyTimestamp
	if (ldap_search_s(conn->ld, conn->searchbase, LDAP_SCOPE_ONELEVEL, conn->filter, attrs, 0, &res)) {
		ldap_debug(conn, 0, "Unable to search on %s", conn->searchbase);
		return NULL;
	}

	res2 = ldap_first_entry(conn->ld, res);
	if (!res2) {
		ldap_debug(conn, 2, "No entries found");
		return NULL;
	}

	do {
		//We have a new entry
		entry = g_malloc0(sizeof(struct data_entry));

		entry->entryUUID = (ldap_get_values(conn->ld, res2, "entryUUID"))[0];
		entry->modifyTimestamp = (ldap_get_values(conn->ld, res2, "modifyTimestamp"))[0];
		entry->uid = quoted_encode(ldap_get_dn(conn->ld, res2));
		//entry->uid = quoted_encode((ldap_get_values(conn->ld, res2, "cn"))[0]);
		ldap_debug(conn, 3, "Loaded entry: %s, %s, %s", entry->entryUUID, entry->modifyTimestamp, entry->uid);

		headlist = g_list_append(headlist, entry);
		res2 = ldap_next_entry(conn->ld, res2);
	} while (res2);

	ldap_debug(conn, 3, "end: load_ldap_entries");
	return headlist;
}

/*Get the complete data for the given head entry*/
/*returns them as LDAPMod*/
void get_ldap_data(ldap_connection *conn, struct data_entry *ldapentry)
{
	char filter[1024];
	LDAPMessage *res = NULL, *res2 = NULL, *res3 = NULL;
	char *attrs[] = {"*", NULL};
	int count = 0, n = 0;
	char **ret = NULL;
	BerElement *berptr = NULL;
	char *attribute = NULL;

	ldap_debug(conn, 2, "Loading full data for: %s", ldapentry->uid);

	ldapentry->ldapdata = g_malloc0(1024 * sizeof(LDAPMod *));

	sprintf(filter, "(entryUUID=%s)", ldapentry->entryUUID);
	if (ldap_search_s(conn->ld, conn->searchbase, LDAP_SCOPE_ONELEVEL, filter, attrs, 0, &res)) {
		ldap_debug(conn, 0, "Unable to search");
		return;
	}

	res2 = ldap_first_entry(conn->ld, res);
	if (!res2) {
		ldap_debug(conn, 0, "No ldap entry returned!: %s", ldapentry->entryUUID);
		return;
	}

	attribute = ldap_first_attribute(conn->ld, res2, &berptr);
	count = 0;
	while (attribute) {
		//Found a new attribute
		ldapentry->ldapdata[count] = malloc(sizeof(LDAPMod));
		ldapentry->ldapdata[count]->mod_values = ldap_get_values(conn->ld, res2, attribute);
		ldapentry->ldapdata[count]->mod_type = attribute;
		ldap_debug(conn, 4, "Attribute: %s=%s", attribute, ldapentry->ldapdata[count]->mod_values[0]);

		attribute = ldap_next_attribute(conn->ld, res2, berptr);
		count++;
	}
	ldap_debug(conn, 3, "end: get_ldap_data");
}

/*Add a ldap_data struct to the ldap server*/
int ldap_add_entry(ldap_connection *conn, LDAPMod **data, char *uidret, int duplicate)
{
	int i = 0, x = 0, n = 0, result = 0;
	char dn[1024];
	ldap_debug(conn, 3, "start: ldap_add_entry");

	for (i = 0; data[i]; i++) {
		//Add another attribute
		data[i]->mod_op = LDAP_MOD_ADD;
		if (!strcmp(data[i]->mod_type, "cn") && !duplicate) {
			sprintf(dn, "cn=3D%s,%s", quoted_encode(data[i]->mod_values[0]), quoted_encode(conn->searchbase));
			strcpy(uidret, dn);
		}
	}

	if (duplicate) {
		sprintf(dn, "uid=3Dduplicate%d,%s",  rand(), quoted_encode(conn->searchbase));
		strcpy(uidret, dn);
		data[i] = g_malloc0(sizeof(LDAPMod));
		data[i]->mod_values = g_malloc0(2 * sizeof(char *));
		data[i]->mod_type = "uid";
		data[i]->mod_values[0] = ldap_explode_dn(quoted_decode(dn), 1)[0];
		data[i]->mod_values[1] = NULL;
		data[i]->mod_op = LDAP_MOD_ADD;
		data[i + 1] = NULL;
	}

	ldap_debug(conn, 2, "Adding: %s", quoted_decode(dn));
	if (result = ldap_add_s(conn->ld, quoted_decode(dn), data)) {
		if (result == 68 && !duplicate) {
			//Multisync wants to duplicate item
			ldap_debug(conn, 2, "Duplicating Entries");
			result = ldap_add_entry(conn, data, uidret, 1);
			return result;
		}
		ldap_debug(conn, 1, "Add result: %i: %s", result, ldap_err2string(result));
		return -1;
	}

	ldap_debug(conn, 3, "end: ldap_add_entry");
	return 0;
}

int ldap_modify_entry(ldap_connection *conn, LDAPMod **data)
{
	int i = 0, x = 0;
	ldap_debug(conn, 3, "start: ldap modify entry");
	for (i = 0; data[i]; i++) {
		for (x = 0; data[i]->mod_values[x]; x++) {
			ldap_debug(conn, 2, "%s=%s\n", data[i]->mod_type, data[i]->mod_values[x]);
		}
	}
	ldap_debug(conn, 3, "end: ldap modify entry");
	return 0;
}

int ldap_delete_entry(ldap_connection *conn, char *cn)
{
	int result = 0;
	ldap_debug(conn, 2, "Deleting %s", cn);

	if (result = ldap_delete_s(conn->ld, cn)) {
		ldap_debug(conn, 1, "Delete result: %s", ldap_err2string(result));
		return -1;
	}
	ldap_debug(conn, 3, "end: ldap_delete_entry");
	return 0;
}
