#include "ldap_plugin.h"

char *vcardattrs[][3] = {
//Normal Stuff
{"TEL;HOME", "homePhone", NULL},
{"TEL;WORK", "companyPhone", NULL},
{"X-EVOLUTION-FILE-AS", "fileAs", NULL},
{"ORG", "o", NULL},
{"URL", "labeledURI", NULL},
{"EMAIL;INTERNET", "mail", NULL},
{"BDAY", "birthDate", NULL},
{"TEL;ISDN", "internationaliSDNNumber", NULL},
{"CATEGORIES", "category", NULL},
{"ROLE", "businessRole", NULL},
{"X-EVOLUTION-SPOUSE", "spouseName", NULL},
{"X-EVOLUTION-ASSISTANT", "assistantName", NULL},
{"X-EVOLUTION-MANAGER", "managerName", NULL},
{"X-EVOLUTION-ANNIVERSARY", "anniversary", NULL},
{"TEL;X-EVOLUTION-CALLBACK", "callbackPhone", NULL},
{"TEL;X-EVOLUTION-ASSISTANT", "assistantPhone", NULL},
{"TEL;PAGER", "pager", NULL},
{"TEL;PREF", "primaryPhone", NULL},
{"TEL;X-EVOLUTION-RADIO", "radio", NULL},
{"TEL;X-EVOLUTION-TELEX", "telex", NULL},
{"TEL;X-EVOLUTION-TTYTDD", "tty", NULL},
{"X-EVOLUTION-OFFICE", "roomNumber", NULL},
{"NICKNAME", "displayName", NULL},
{"TEL;WORK;VOICE", "telephoneNumber", NULL},
{"TEL;CELL", "mobile", NULL},
{"TEL;CAR", "carPhone", NULL},
{"TEL;WORK;FAX", "facsimileTelephoneNumber", NULL},
{"TEL;HOME;FAX", "homeFacsimileTelephoneNumber", NULL},
{"TEL;VOICE", "otherPhone", NULL},
{"TEL;FAX", "otherFax", NULL},
{"CALURI", "calenderURI", NULL},
{"FBURL", "freeBusyURI", NULL},
{"X-MOZILLA-HTML", NULL, NULL},
{"TITLE", "title", NULL},
//Entries needing quoted-printable TODO!
//Should be made by autodetection
{"NOTE;ENCODING=QUOTED-PRINTABLE", "note", "1"},
{"ADR;HOME;PREF;ENCODING=QUOTED-PRINTABLE", "postalAddress", "1"},
{"ADR;WORK;PREF;ENCODING=QUOTED-PRINTABLE", "homePostalAddress", "1"},
//Special Entries or entries being ignored
{"N.Family", NULL, NULL},
{"N.Given", NULL, NULL},
{"N.Middle", NULL, NULL},
{"N", NULL, NULL},
{"", "objectClass", NULL},
{"FN", NULL, NULL},
{"BEGIN", NULL, NULL},
{"END", NULL, NULL},
{"VERSION", NULL, NULL},
{"UID", NULL, NULL},
NULL};

char *quoted_encode(char *string) {
	char *data;
	char buffer[1024];
	int i = 0;
	int chr = 0;
	data = malloc(1024 * sizeof(char));

	strcpy(data, string);
	for (i = 0; i < strlen(data); i++) {
		if (data[i] == '=') {
			data[i] = 0;
			sprintf(buffer, "%s%s%s", data, "=3D", &data[i + 1]);
			i = i + 2;
			strcpy(data, buffer);
		}
	}

	while(1) {
		//Found new char to encode
		i = strcspn(data, "\n\r ");
		chr = data[i];
		data[i] = 0;
		if (!chr) { break; }
		sprintf(buffer, "%s=%02X%s", data, chr, &data[i + 1]);
		strcpy(data, buffer);
	}
	return data;
}

char *quoted_decode(char *string)
{
	int i = 0;
	char *data;
	char hex[1024];
	char buffer[1024];
	data = malloc(1024 * sizeof(char));
	strcpy(data, string);

	while(1) {
		//Found new char to decode
		i = i + strcspn(&data[i], "=");
		if (!data[i]) { break; }
		data[i] = 0;

		strcpy(hex, "0x");
		strncat(hex, &data[i + 1], 2);

		sprintf(buffer, "%s%c%s", data, ((int)(strtod(hex, NULL))), &data[i + 3]);
		strcpy(data, buffer);
		i++;
	}
	return data;
}

void ldap2vcard(ldap_connection *conn, LDAPMod **ldapdata, GString *vcard)
{
	char vcardstr[2048]; //TODO!
	int i = 0, n = 0, x = 0;
	char var1[1024], var2[1024], var3[1024], var4[1024], var5[1024];
	char *title = "", *first = "", *middle = "", *last = "", *suffix = "";
	GString *categories = g_string_new("");

	ldap_debug(conn, 2, "Converting ldap to vcard");

	g_string_append(vcard, "BEGIN:VCARD\r\n");
	g_string_append(vcard, "VERSION:2.1\r\n");

	//Make the name from the ldapdata
	//g_string_append(vcard, ldapGetName);

	//Walk through all the ldap attributes
	for (n = 0; ldapdata[n]; n++) {
		for (x = 0; ldapdata[n]->mod_values[x]; x++) {
			i = 0;
			ldap_debug(conn, 4, "Converting: %s=%s", ldapdata[n]->mod_type, ldapdata[n]->mod_values[x]);
			//Name entries
			if (!strcmp(ldapdata[n]->mod_type, "cn")) {
				switch (sscanf(ldapdata[n]->mod_values[x], "%s %s %s %s %s", var1, var2, var3, var4, var5)) {
					case 1:
						last = var1;
						break;
					case 2:
						first = var1;
						last = var2;
						break;
					case 3:
						first = var1;
						middle = var2;
						last = var3;
						break;
					case 4:
						title = var1;
						first = var2;
						middle = var3;
						last = var4;
						break;
					case 5:
						title = var1;
						first = var2;
						middle = var3;
						last = var4;
						suffix = var5;
						break;
				}
				goto next;
			}
			if (!strcmp(ldapdata[n]->mod_type, "givenName")) {
				if (strcmp(ldapdata[n]->mod_values[x], first)) {
					ldap_debug(conn, 1, "First names do not match");
					first = ldapdata[n]->mod_values[x];
				}
				goto next;
			}
			if (!strcmp(ldapdata[n]->mod_type, "sn")) {
				if (strcmp(ldapdata[n]->mod_values[x], last)) {
					ldap_debug(conn, 1, "Last names do not match");
					last = ldapdata[n]->mod_values[x];
				}
				goto next;
			}
			//Categories entry
			if (!strcmp(ldapdata[n]->mod_type, "categories")) {
				if (categories->len) {
					g_string_append(categories, ", ");
				}
				g_string_append(categories, ldapdata[n]->mod_values[x]);
				goto next;
			}
			//Normal entries
			while (vcardattrs[i][0]) {
				if (vcardattrs[i][1]) {
					if (strcmp(ldapdata[n]->mod_type, vcardattrs[i][1]) == 0) {
						if (vcardattrs[i][0][0]) {
							//Found the entry in our array. Now convert it
							if (vcardattrs[i][2]) {
								//printf("Multi-Line\n");
								ldapdata[n]->mod_values[x] = quoted_encode(ldapdata[n]->mod_values[x]);
							}
							sprintf(vcardstr, "%s:%s\r\n", vcardattrs[i][0], ldapdata[n]->mod_values[x]);
							g_string_append(vcard, vcardstr);
						}
						goto next;
					}
				}
				i++;
			}

			//We could not find the attribute in our database.
			ldap_debug(conn, 1, "Could not match ldapattribute: %s", ldapdata[n]->mod_type);
			next:
			;
		}
	}
	//Make Name
	snprintf(vcardstr, 2048, "N:%s;%s;%s;%s;%s\r\n", last, first, middle, title, suffix);
	ldap_debug(conn, 2, "Got Name: %s", vcardstr);
	g_string_append(vcard, vcardstr);

	//Categories
	g_string_append(vcard, categories->str);
	g_string_append(vcard, "\r\n");

	g_string_append(vcard, "END:VCARD\r\n");
	ldap_debug(conn, 4, "VCARD:\n%s", vcard->str);
}

/*Convert a vcard to a LDAPMod Struct array*/
LDAPMod **vcard2ldap(ldap_connection *conn, char *origvcard)
{
	int i = 0, n = 0, x = 0, count = 0;
	char attributes[1024];
	GString *name = g_string_new("");
	LDAPMod **ldapentries = NULL;
	char *vcardline, *valueline, value[1024], *vcardname;
	char *vcard;
	char *first = NULL, *last = NULL, *middle = NULL, *prefix = NULL, *suffix = NULL;

	ldap_debug(conn, 2, "converting vcard to ldap");
	ldapentries = g_malloc0(1024 * sizeof(LDAPMod *));
	vcard = g_malloc0(sizeof(origvcard));
	vcard = strdup(origvcard);

	//Convert vcard to array then parse it
	vcardline = strtok(vcard, "\n");
	while (vcardline) {
		//Parse the string into vcard_entries
		ldap_debug(conn, 4, "VCARD: %s", vcardline);
		if (sscanf(vcardline, "%[^:]:%[^\r]", attributes, value) != 2) {
			goto next;
		}

		//Name attributes
		if (!strcmp(attributes, "N")) {
			vcardname = strdup(value);
			ldap_debug(conn, 3, "Found Name: %s", vcardname);
			valueline = strsep(&vcardname, ";");
			i = 0;
			while (valueline) {
				if (valueline[0]) {
					switch (i) {
						case 0:
							last = valueline;
							break;
						case 1:
							first = valueline;
							break;
						case 2:
							middle = valueline;
							break;
						case 3:
							prefix = valueline;
							break;
						case 4:
							suffix = valueline;
							break;
					}
				}
				i++;
				valueline = strsep(&vcardname, ";");
			}
			if (prefix) {
				g_string_append(name, prefix);
			}
			if (first) {
				ldapentries[count] = malloc(sizeof(LDAPMod));
				ldapentries[count]->mod_type = "givenName";
				ldapentries[count]->mod_values = malloc(2 * sizeof(char *));
				ldapentries[count]->mod_values[0] = strdup(first);
				ldapentries[count]->mod_values[1] = NULL;
				count++;
				if (name->len) {
					g_string_append(name, " ");
				}
				g_string_append(name, first);
			}
			if (middle) {
				if (name->len) {
					g_string_append(name, " ");
				}
				g_string_append(name, middle);
			}
			if (last) {
				ldapentries[count] = malloc(sizeof(LDAPMod));
				ldapentries[count]->mod_type = "sn";
				ldapentries[count]->mod_values = malloc(2 * sizeof(char *));
				ldapentries[count]->mod_values[0] = strdup(last);
				ldapentries[count]->mod_values[1] = NULL;
				count++;
				if (name->len) {
					g_string_append(name, " ");
				}
				g_string_append(name, last);
			} else {
				ldap_debug(conn, 0, "Wrong N attribute");
			}
			if (suffix) {
				if (name->len) {
					g_string_append(name, " ");
				}
				g_string_append(name, suffix);
			}
			//add it
			ldapentries[count] = malloc(sizeof(LDAPMod));
			ldapentries[count]->mod_type = "cn";
			ldapentries[count]->mod_values = malloc(2 * sizeof(char *));
			ldapentries[count]->mod_values[0] = strdup(name->str);
			ldapentries[count]->mod_values[1] = NULL;
			count++;
			goto next;
		}

		//Categories entry
		if (!strcmp(attributes, "CATEGORIES")) {
			x = 0;
			ldapentries[count] = malloc(sizeof(LDAPMod));
			ldapentries[count]->mod_type = "categories";
			ldapentries[count]->mod_values = malloc(1024 * sizeof(char *));

			vcardname = value;
			for (i = 0; value[i]; i++) {
				if (value[i] == ',') {
					value[i] = 0;
					ldapentries[count]->mod_values[x] = strdup(vcardname);
					ldapentries[count]->mod_values[x + 1] = NULL;
					vcardname = &value[i + 2];
					x++;
				}
			}
			ldapentries[count]->mod_values[x] = strdup(vcardname);
			ldapentries[count]->mod_values[x + 1] = NULL;
			count++;
			goto next;
		}

		//Normal entries
		//Search for the attribute in vcardattrs
		i = 0;
		while (vcardattrs[i][0]) {
			if (!strcmp(attributes, vcardattrs[i][0])) {
				//Found the entry in our array.
				if (vcardattrs[i][1]) {
					// It needs to be converted
					//Search for the attribute in the existing LDAPMod array
					n = 0;
					while (ldapentries[n]) {
						if (!strcmp(ldapentries[n]->mod_type, vcardattrs[i][1])) {
							//Found existing LDAPMod entry
							x = 0;
							while (ldapentries[n]->mod_values[x]) {
								x++;
							}
							if (vcardattrs[i][2]) {
								ldap_debug(conn, 4, "Multi-Line");
								ldapentries[n]->mod_values[x] = quoted_decode(value);
							} else {
								ldapentries[n]->mod_values[x] = strdup(value);
							}
							ldapentries[n]->mod_values[x + 1] = NULL;
							//next vcard part
							goto next;
						}
						n++;
					}

					//Now convert it to a new LDAPMod entry
					ldapentries[count] = malloc(sizeof(LDAPMod));
					ldapentries[count]->mod_type = strdup(vcardattrs[i][1]);
					ldapentries[count]->mod_values = malloc(1024 * sizeof(char *));
					if (vcardattrs[i][2]) {
						ldap_debug(conn, 4, "Multi-Line");
						ldapentries[count]->mod_values[0] = quoted_decode(value);
					} else {
						ldapentries[count]->mod_values[0] = strdup(value);
					}
					ldapentries[count]->mod_values[1] = NULL;
					count++;
				}
				goto next;;
			}
			i++;
		}
		//We could not find the attribute in our database.
		ldap_debug(conn, 0, "Could not match vcard attribute: %s = %s. Possible dataloss!", attributes, value);

		next:
		vcardline = strtok(NULL, "\n");
	}

	ldapentries[count] = malloc(sizeof(LDAPMod));
	ldapentries[count]->mod_op = 0;
	ldapentries[count]->mod_type = "objectClass";
	ldapentries[count]->mod_values = malloc(6 * sizeof(char *));
	ldapentries[count]->mod_values[0] = "top";
	ldapentries[count]->mod_values[1] = "person";
	ldapentries[count]->mod_values[2] = "organizationalPerson";
	ldapentries[count]->mod_values[3] = "inetOrgPerson";
	ldapentries[count]->mod_values[4] = "evolutionPerson";
	ldapentries[count]->mod_values[5] = NULL;

	count++;
	ldapentries[count] = NULL;

	ldap_debug(conn, 3, "end: vcard2ldap");
	return ldapentries;
}
